;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xb173ba38fc4d3d32d195f10664c3253d . "./code.cg2js/cg-status-bar.fasl")
(#x993de112c02a6e143287783cdcc9784f . "./code.cg2js/cg-hotspot.fasl")
(#x6e6c943b99892ef1cdfdc10c494a387c . "./code.cg2js/cg-toolbar.fasl")
(#x7bf9cf8ad23a2ae50e4389c3c0efc70e . "./code.cg2js/cg-clipboard-stack.fasl")
(#x93920834f5f60d43f6371c0138276176 . "./code.cg2js/cg-tooltip.fasl")
(#x8591ad77214a31ba8e10786a946e6eac . "./code.cg2js/cg-calendar.fasl")
(#x4c96ff10fdf59b7a4bc7666af6b5650c . "./code.cg2js/cg-item-list.fasl")
(#x8370286e3daec721df225c603bd65756 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xd155273dd2ddd5bbd823ef5784c38c4c . "./code.cg2js/cg-trackbar.fasl")
(#xab96c4dea3b55e340fafc7e3e4a75ea7 . "./code.cg2js/cg-class-grid.fasl")
(#xfce77d1fdec8cf02debe331dfe813c9b . "./code.cg2js/cg-dialog-item.fasl")
(#xbe5deb124505b7f4e2aba639d0c75355 . "./code.cg2js/cg-comtab.fasl")
(#xc072da825398ad79461b675691b1588a . "./code.cg2js/cg-lisp-text.fasl")
(#x75ef5e92f8c64522a708f240d0b4fb0e . "./code.cg2js/cg-button.fasl")
(#x4110347640dc040501654ca80241cc13 . "./code.cg2js/cg-printing.fasl")
(#x078570da6068c3e4ed2492186ac0152b . "./code.cg2js/cg-pixmap-printing.fasl")
(#x4357ee0bf7f377e86b02ee8d961d745f . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x219f53aa981a802404a02546f51ee35f . "./code.cg2js/cg-pixmap-widget.fasl")
(#xad4c0ec632aeef63219bbed785ed97be . "./code.cg2js/cg-selected-object.fasl")
(#x3c90fe2df0d28a810e6e5f488a8601c0 . "./code.cg2js/cg-text-or-combo.fasl")
(#xcad40cc0254aaf1dfb297f2557c7fc69 . "./code.cg2js/cg-graphics-context.fasl")
(#x4cef3928a5c7fa4f2ee18cdb7784237b . "./code.cg2js/cg-edit-in-place.fasl")
(#xa3eeec91a521fbbc8a34701c35c7afc1 . "./code.cg2js/cg-string-dialog.fasl")
(#x4aa24e7c440408f64a0237a0004e9ce3 . "./code.cg2js/cg-list-view.fasl")
(#x9d3171366dd7ec7bbc38bbe6ffe8fdc4 . "./code.cg2js/cg-intersect.fasl")
(#x6a7583217ccba3bb125a20190c39b02d . "./code.cg2js/cg-text-widget.fasl")
(#xd848c7155b2b56b285a7f75724cf0519 . "./code.cg2js/cg-directory-dialog.fasl")
(#xba9543fb84c2618164decb503fffddf6 . "./code.cg2js/cg-file-dialog.fasl")
(#xac6238ffbcbcbd0c2669286e150ec7e3 . "./code.cg2js/cg-dde-utils.fasl")
(#x5bb904f8e0b49c4d62465fc943147b27 . "./code.cg2js/cg-nodes.fasl")
(#xceaccc625ecd4abd9c554117ebc91ad8 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xb970838bc69ca19e555091be537f382e . "./code.cg2js/cg-up-down-control.fasl")
(#x54309a26fbbb986e01661f26c7143b0a . "./code.cg2js/cg-get-position.fasl")
(#x109a0add61240242045c505b0d833d29 . "./code.cg2js/cg-toggling-widget.fasl")
(#x2415bd44a2d1aeede3aa9bf04a9d6cac . "./code.cg2js/cg-acache.fasl")
(#x69ff23ade3a8765b639377aafbb81ef8 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x959cd2bdb0bc8904c14ec4c4c6099b43 . "./code.cg2js/cg-base.fasl")
(#x48a9b28103ab833b9733a08d34fb59d8 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x2dd256800447f689416220ec0d6b505e . "./code.cg2js/cg-find-string-dialog.fasl")
(#x5780a1e5d417a9faa8ccc42bef7857e0 . "./code.cg2js/cg-property.fasl")
(#xf026ddd691f4b3babe62b6573666bcac . "./code.cg2js/cg-editable-text.fasl")
(#x8f64c3e7db6fbd9063c949fdeead4831 . "./code.cg2js/cg-lamp.fasl")
(#x547d1a962653abd03cd0b3a729c02b25 . "./code.cg2js/cg-progress-indicator.fasl")
(#x60d262dcfdcd23679c469b3aafdb02a0 . "./code.cg2js/cg-chart-widget.fasl")
(#xd7c01ee4c9a887febf931f3a6fc427bd . "./code.cg2js/cg-common-control.fasl")
(#xf63483598a652552c47f00151d88d6c7 . "./code.cg2js/cg-font-dialog.fasl")
(#x22c7c56e02461d5a3907675d7bbf77d8 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xf2891f7cf13676563d5e9e1a2552f608 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x7864441a634a0322eda1f3220a6a4e6d . "./code.cg2js/cg-os-window.fasl")
(#xbf65e1de76be1698af73d80e51c86980 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x886ff2518d2874c9de9a1258799832f0 . "./code.cg2js/cg-pixmap.fasl")
(#xc20b1a0bd91d7128b78cd428808320e2 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xd994ee5eed48fd56803896a7b1818ab8 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x6caeaabfc3b589963edeb9ddb8f2c6c2 . "./code.cg2js/cg-scaling-stream.fasl")
(#xdca106783c02bd9f2bca53c78b021387 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x82a651ee91a7e7e89a735f63f1b15452 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x3cf9c01e379360a01aa81f312f11d44f . "./code.cg2js/cg-timer.fasl")
(#x53b7eca6d6e2d5a66b23fe6b0df99f8c . "./code.cg2js/cg-scroll-bar.fasl")
(#x806b3d7ae28096174b43e32c99ffebc7 . "./code.cg2js/cg-caret.fasl")
(#xe700a242edd89491539a4800c26360bd . "./code.cg2js/cg-template-string.fasl")
(#x87578e5328acae4456c0a9741ced053a . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xf828277a7f0537bb332134124e43f318 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe9b29aaa13e6957663eda1ed9d0dddc8 . "./code.cg2js/cg-combo-box.fasl")
(#x91774a38c879f024b04073cb6ce1ccf4 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x544aed7b1bfce8d3f10ebf524851a732 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xe2592612b6a57a6b1bd06309fbfbea03 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x39cdb5097b0dd452c6b429b9b50399d2 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x764905939ded01de0dc87885f2f3c9f3 . "./code.cg2js/cg-player.fasl")
(#x776026787e7cfabd4e5117ddb6b62d70 . "./code.cg2js/cg-outline.fasl")
(#x77a9be431d1ce34aa9ac5cbbc04dc7fc . "./code.cg2js/cg-class-support.fasl")
(#x0b4d0a0726fc77a8023a038b857fb55e . "./code.cg2js/cg-lettered-menu.fasl")
(#xad1b6eeacfe4bc43ab3afc12120f275b . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xcfad883f82dd53a661baf3d0542600eb . "./code.cg2js/cg-fill-texture.fasl")
(#x8ccdf640f07d4e9792a16343a62f0d5d . "./code.cg2js/cg-lisp-widget.fasl")
(#x3fdccd98f20d0e13b5ca3888b9d63f11 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xcb639a745735c9850d9d2f0b6530b8a7 . "./code.cg2js/cg-clipboard.fasl")
(#x2c4a11ce79d20dfc1861d3199fc27451 . "./code.cg2js/cg-wrap-string.fasl")
(#xd89fd13722e93857838a4247614b29a8 . "./code.cg2js/cg-os-widget.fasl")
(#xffd1e1057e1f02d6381142df6294e221 . "./code.cg2js/cg-check-box.fasl")
(#xf93cce28c12f8ad456f92de2069a851f . "./code.cg2js/cg-choice-list.fasl")
(#xe1cd980c376cb9cf40f0e13e0a7237f0 . "./code.cg2js/cg-layout.fasl")
(#x926676d24785dd0ddd7470efd4ec884d . "./code.cg2js/cg-group-box.fasl")
(#xbe4e59b310224af5f4169e65c9b48d44 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x3668fc8a33ee5dda2496a0e43956ae74 . "./code.cg2js/cg-radio-button.fasl")
(#xb832c24797896e0d10cb63388c9e9e03 . "./code.cg2js/cg-tab-control.fasl")
(#x2e74775ecf8662bb3609368574f077c6 . "./code.cg2js/cg.fasl")
(#x671532cbd932af257b8968dbc5df7156 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xaed5af118e5c0fccc9aaeb3e50577ff7 . "./code.cg2js/cg-dropping-outline.fasl")
(#x2198761c7231719c45c0dc410df549a6 . "./code.cg2js/cg-get-pixmap.fasl")
(#x28b7614df3f351a95abb3393635750d8 . "./code.cg2js/cg-grid-widget.fasl")
(#xc8f7b4deccbb7bc50c2cc8037db049d4 . "./code.cg2js/cg-utility-dialog.fasl")
(#x280729f87870acd31c84d72a3d644541 . "./code.cg2js/cg-static-text.fasl")
(#x9cbaa094148740ba3de95ef71a71d3e1 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xe8b9bd641b424d5170c479d83f35a744 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xff00c06bb621c5ae9e8337567cad6b02 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xf77c0bb39635c452afea2673ff5d47fc . "./code.cg2js/cg-bitmap-pane.fasl")
(#x27b29b7d8e90c5fd8f04eaf5ba75982f . "./code.cg2js/cg-plot-widget.fasl")
(#xdcc5d21889bf99e635b83376a536a178 . "./code.cg2js/cg-color-dialog.fasl")
(#xad0adc3682326d4cc00ed9e55307da30 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xa16bd967a4a7c6ce34cba5769331fc35 . "./code.cg2js/cg-icon.fasl")
(#xddb1a9a214cbf10b350d38218726dda8 . "./code.cg2js/cg-dev.fasl")
(#xb5844567877bae9490c1414174f1630c . "./code.cg2js/cg-choose-printer.fasl")
(#x7bb372cc0632f297ce922a355d7dbe29 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xcc170d8e1104a2cb825f590bd6d2e0c4 . "./code.cg2js/cg-menu.fasl")
(#x90481b56bfc66a57ab85024a9411367b . "./code.cg2js/cg-yes-no-list.fasl")
(#xf24e1f78351557b3d4166340136b75b9 . "./code.cg2js/cg-palette.fasl")
(#x0bef7283be897d20852e7e013b262df2 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xc1e75298185390a9ba0597da85d89862 . "./code.cg2js/cg-message-dialog.fasl")
(#xb4fdf3c25985c26e7be5122182a21b8a . "./code.cg2js/cg-shortcut-menu.fasl")
(#xb22a6089854e8cc36fbec364d30352d2 . "./code.cg2js/cg-split-bar.fasl")
(#x035e1d86bf9ee1fa32ca18e568e0e9ad . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x9c5daf133c9165ade0f00a973be0c14d . "./code.cg2js/cg-web-browser.fasl")
(#xac0bab82c5b9af2ce59c5d7cde5316ff . "./code.cg2js/cg-project-window.fasl")
(#x86ad4e6f747a8618ea33922a1d303ac1 . "./code.cg2js/cg-picture-widget.fasl")
(#xd3bc0e91fc78ff7e43cb714c59201e84 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xc324d04de8d3400cb9186250acad1bc3 . "./code.cg2js/cg-object-editor.fasl")
(#xe47b088d58ef2e1471cc0760f6600cab . "./code.cg2js/cg-paren-matching.fasl")
(#xe6d69bb05e4fec412da907caf8e9ecab . "./code.cg2js/cg-sample-file-menu.fasl")
(#xf7b005f81c6bd7495052c330338d3b11 . "./code.cg2js/cg-drawable.fasl")
(#x085af69301d64a653319c3dd1d31d73f . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x429a82e900a9010565d506f41af11081 . "./code.cg2js/cg-html-widget.fasl")
(#x901fd9e3a418ca0661e18053ed7093c4 . "./code.cg2js/cg-curve.fasl")
(#x123328d623e3225567bb2df3bb09557f . "./code.cg2js/cg-yes-no-string.fasl")
(#x06d358d35e443ca492437aeda1141bc4 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x47954fb295020eadf4c2670fc2d22216 . "./code.cg2js/cg-html-dialog.fasl")
(#xe73440cd52a5b89eb00fdbec2f5c3a11 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xb156157a145384d8ccf3e4750914321f . "./code.cg2js/cg-tray.fasl")
(#xc7269abfba8a2df72fea1ac65b86da77 . "./code.idejs/ide-apropos.fasl")
(#xf7c7d64c778480d79537fed658008a65 . "./code.idejs/ide-window-list.fasl")
(#x8ec41e41f78f2fd382a37469f7f19e75 . "./code.idejs/ide-class-browser.fasl")
(#x14b77f3bb8729a77861ea35b5e3c0cfe . "./code.idejs/ide-base.fasl")
(#x4e7698514faceb3a63515d5631edb5ea . "./code.idejs/ide-process-browser.fasl")
(#xffb7e2cb324b0c8264f875c87a086f29 . "./code.idejs/ide-acache.fasl")
(#x84ba9b38c8ba4d28b85d874092ddd778 . "./code.idejs/ide-navigator.fasl")
(#x3bea3325cd9a1a25d3a7a7d419722afc . "./code.idejs/ide-trace.fasl")
(#x8c8f19325caef3a3327043d66b43dcb2 . "./code.idejs/ide-project.fasl")
(#x73575b55d9f87f71637871ddf78158ad . "./code.idejs/ide-patch-dialog.fasl")
(#x031db0b7395d392673a59388f5e660b4 . "./code.idejs/ide-grapher.fasl")
(#x8c972fde40a8047430abcc7e7e26a588 . "./code.idejs/ide-dev.fasl")
(#x0c51f71ba3f23d437cd51f7e6301ec4c . "./code.idejs/ide-keyshort.fasl")
(#xe93901d03281c6ef9fcc7a0a47b39986 . "./code.idejs/ide-editor.fasl")
(#xda947e8a9cf90ade35a05fee12e56d81 . "./code.idejs/ide-inspector.fasl")
(#x4f4cea3ddb079c9a8aa2a52927402410 . "./code.idejs/ide.fasl")
(#x267bdf99877820a535d644cbdf274f5c . "./code.idejs/ide-clipboard.fasl")
(#x5e4ff5253551535e92f1bfc96a298b12 . "./code.idejs/ide-find-definitions.fasl")
(#x5d77d1be5f7d3435d5152c48132e17af . "./code.idejs/ide-form.fasl")
(#xc36284acf9e497a8275bc63c37477df2 . "./code.idejs/ide-intro.fasl")
(#x115c6737eae3a7dfc0f75b082b0f03a6 . "./code.idejs/ide-find-in-files.fasl")
(#xc881caee18d42bb09956dd7847e1ad6c . "./code.idejs/ide-package-list.fasl")
(#xd678fe385fcbea03519208c45a46d1b8 . "./code.idejs/ide-class-grapher.fasl")
(#x4a9bad5a7becdca3247e09ac504560a1 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
